package alterrs.jbct.deob;

import static org.objectweb.asm.Opcodes.BIPUSH;
import static org.objectweb.asm.Opcodes.ICONST_0;
import static org.objectweb.asm.Opcodes.ICONST_1;
import static org.objectweb.asm.Opcodes.ICONST_2;
import static org.objectweb.asm.Opcodes.ICONST_3;
import static org.objectweb.asm.Opcodes.ICONST_4;
import static org.objectweb.asm.Opcodes.ICONST_5;
import static org.objectweb.asm.Opcodes.ICONST_M1;
import static org.objectweb.asm.Opcodes.ISHL;
import static org.objectweb.asm.Opcodes.ISHR;
import static org.objectweb.asm.Opcodes.IUSHR;
import static org.objectweb.asm.Opcodes.LDC;
import static org.objectweb.asm.Opcodes.LSHL;
import static org.objectweb.asm.Opcodes.LSHR;
import static org.objectweb.asm.Opcodes.LUSHR;
import static org.objectweb.asm.Opcodes.SIPUSH;

import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.InsnNode;
import org.objectweb.asm.tree.IntInsnNode;
import org.objectweb.asm.tree.LdcInsnNode;
import org.objectweb.asm.tree.MethodNode;

import alterrs.jbct.utility.ASMUtil;
import alterrs.jbct.utility.AbstractTransformer;

/**
 * 
 * @author Kyle Friz
 * @since  Apr 27, 2016
 */
public class BitShiftDeobfuscator extends AbstractTransformer {

	private int count = 0;

	public BitShiftDeobfuscator() {
		super(true);
	}

	@Override
	public void visitInsn(ClassNode c, MethodNode m, AbstractInsnNode n) {
		if (n.getOpcode() == ISHL || n.getOpcode() == ISHR || n.getOpcode() == IUSHR
				|| n.getOpcode() == LSHL || n.getOpcode() == LSHR || n.getOpcode() == LUSHR) {
			
			AbstractInsnNode push = ASMUtil.previousRealInsn(n.getPrevious());

			if (push == null || !isConst(push))
				return;
			
			long val = getPush(push);
			
			if (n.getOpcode() == ISHL || n.getOpcode() == ISHR || n.getOpcode() == IUSHR) {
				long valv = val & 0x1F;
				if (val != valv) {
					m.instructions.set(push, createPush(valv));
					count++;
				}
			} else {
				long valv = val & 0x3F;
				if (val != valv) {
					m.instructions.set(push, createPush(valv));
					count++;
				}
			}
		}
	}

	private boolean isConst(AbstractInsnNode n) {
		switch (n.getOpcode()) {
		case BIPUSH:
		case SIPUSH:
		case LDC:
		case ICONST_M1:
		case ICONST_0:
		case ICONST_1:
		case ICONST_2:
		case ICONST_3:
		case ICONST_4:
		case ICONST_5:
			return true;
		}
		return false;
	}
	
	private AbstractInsnNode createPush(Number num) {
		long value = num.longValue();
		if (value == -1) {
			return new InsnNode(ICONST_M1);
		} else if (value == 0) {
			return new InsnNode(ICONST_0);
		} else if (value == 1) {
			return new InsnNode(ICONST_1);
		} else if (value == 2) {
			return new InsnNode(ICONST_2);
		} else if (value == 3) {
			return new InsnNode(ICONST_3);
		} else if (value == 4) {
			return new InsnNode(ICONST_4);
		} else if (value == 5) {
			return new InsnNode(ICONST_5);
		} else if (value >= Byte.MIN_VALUE && value <= Byte.MAX_VALUE) {
			return new IntInsnNode(BIPUSH, (int) value);
		} else if (value >= Short.MIN_VALUE && value <= Short.MAX_VALUE) {
			return new IntInsnNode(SIPUSH, (int) value);
		} else if (value >= Integer.MIN_VALUE && value <= Integer.MAX_VALUE) {
			return new LdcInsnNode((int) value);
		} else {
			return new LdcInsnNode(/* (long) */value);
		}
	}

	private long getPush(AbstractInsnNode push) {
		if (push instanceof InsnNode) {
			switch (push.getOpcode()) {
			case ICONST_M1:
				return -1;
			case ICONST_0:
				return 0;
			case ICONST_1:
				return 1;
			case ICONST_2:
				return 2;
			case ICONST_3:
				return 3;
			case ICONST_4:
				return 4;
			case ICONST_5:
				return 5;
			default:
				throw new AssertionError();
			}
		} else if (push instanceof IntInsnNode) {
			return ((IntInsnNode) push).operand;
		} else {
			return ((Number) ((LdcInsnNode) push).cst).longValue();
		}
	}

	@Override
	public void onFinish() {
		System.out.println(" ^	Simplified " + count + " bit shifts!");
	}
}
