package alterrs.jbct.deob;

import static org.objectweb.asm.Opcodes.ACC_FINAL;
import static org.objectweb.asm.Opcodes.INVOKESPECIAL;
import static org.objectweb.asm.Opcodes.PUTFIELD;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.FieldInsnNode;
import org.objectweb.asm.tree.FieldNode;
import org.objectweb.asm.tree.InsnList;
import org.objectweb.asm.tree.MethodNode;

import alterrs.jbct.JBCT;
import alterrs.jbct.utility.AbstractTransformer;
import alterrs.jbct.utility.FieldID;

/**
 * @author Lazaro Brito
 */
public class FieldDeobfuscator extends AbstractTransformer {

	private int count = 0, count2 = 0;

	public FieldDeobfuscator() {
		super(true);
	}

	@Override
	public void visitMethod(ClassNode c, MethodNode m) {
		if (m.name.equals("<init>")) {
			AbstractInsnNode n = m.instructions.getFirst();
			while (n != null) {
				if (n.getOpcode() == PUTFIELD) {
					List<AbstractInsnNode> block = new ArrayList<AbstractInsnNode>();

					AbstractInsnNode n2 = m.instructions.getFirst();
					while (n2 != null) {
						block.add(n2);

						if (n2 == n) {
							break;
						}
						n2 = n2.getNext();
					}

					for (AbstractInsnNode n3 : block) {
						m.instructions.remove(n3);
					}

					n2 = m.instructions.getFirst();
					while (n2 != null && n2.getOpcode() != INVOKESPECIAL) {
						n2 = n2.getNext();
					}

					InsnList list = new InsnList();
					for (AbstractInsnNode n3 : block) {
						list.add(n3);
					}

					m.instructions.insert(n2, list);
					count2++;
					break;
				} else if (n.getOpcode() == INVOKESPECIAL) {
					return;
				}

				n = n.getNext();
			}
		}
	}

	@Override
	public void visitFieldInsn(ClassNode c, MethodNode m, FieldInsnNode n) {
		FieldNode f = JBCT.ctx.module.field(new FieldID(n));
		if (f != null && (f.access & ACC_FINAL) != 0) {
			f.access &= ~ACC_FINAL;

			count++;
		}
	}

	@Override
	public void onFinish() {
		System.out.println(" ^	Definalized " + count + " fields!");
		System.out.println(" ^	Reorganized " + count2 + " put-fields!");
	}
}
