package com.method.rscd;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Proxy;
import java.net.SocketAddress;
import java.net.Proxy.Type;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class ProxyList {

	private static final Set<ProxyEndpoint> proxies = new HashSet<>();

	public static void load(String path) {
		File file = new File(path);
		if (!file.exists() || file.isDirectory()) {
			throw new IllegalArgumentException("The specified path does not exist or is not a file.");
		}
		try (BufferedReader reader = Files.newBufferedReader(file.toPath())) {
			do {
				String line = reader.readLine();
				if (line == null || line.isEmpty()) {
					break;
				}
				String[] parts = line.split(":", 2);
				proxies.add(new ProxyEndpoint(parts[0], Integer.parseInt(parts[1])));
			} while (true);
		} catch (IOException e) {
			System.err.println("Failed to load the proxy list");
			e.printStackTrace();
			System.exit(1);
		}
	}

	public static Set<ProxyEndpoint> getProxies() {
		return proxies;
	}

	public static class ProxyEndpoint {

		public final String host;
		public final int port;

		public ProxyEndpoint(String host, int port) {
			this.host = host;
			this.port = port;
		}

		public Proxy toHttpProxy() {
			SocketAddress addr = new InetSocketAddress(host, port);
			return new Proxy(Type.HTTP, addr);
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((host == null) ? 0 : host.hashCode());
			result = prime * result + port;
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			ProxyEndpoint other = (ProxyEndpoint) obj;
			if (host == null) {
				if (other.host != null)
					return false;
			} else if (!host.equals(other.host))
				return false;
			if (port != other.port)
				return false;
			return true;
		}

	}
}
